package sun.net.ftp;

import java.security.AccessController;
import sun.net.ftp.impl.DefaultFtpClientProvider;
import java.security.PrivilegedAction;
import java.util.ServiceConfigurationError;

public abstract class FtpClientProvider
{
    private static final Object lock;
    private static FtpClientProvider provider;
    
    public abstract FtpClient createFtpClient();
    
    protected FtpClientProvider() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("ftpClientProvider"));
        }
    }
    
    private static boolean loadProviderFromProperty() {
        String property = System.getProperty("sun.net.ftpClientProvider");
        if (property == null) {
            return false;
        }
        try {
            FtpClientProvider.provider = (FtpClientProvider)Class.forName(property, true, null).newInstance();
            return true;
        }
        catch (ClassNotFoundException | IllegalAccessException | InstantiationException | SecurityException ex) {
            throw new ServiceConfigurationError(ex.toString());
        }
    }
    
    private static boolean loadProviderAsService() {
        return false;
    }
    
    public static FtpClientProvider provider() {
        synchronized (FtpClientProvider.lock) {
            if (FtpClientProvider.provider != null) {
                return FtpClientProvider.provider;
            }
            return (FtpClientProvider)AccessController.doPrivileged(new PrivilegedAction<Object>() {
                @Override
                public Object run() {
                    if (FtpClientProvider.access$000()) {
                        return FtpClientProvider.provider;
                    }
                    if (FtpClientProvider.access$200()) {
                        return FtpClientProvider.provider;
                    }
                    FtpClientProvider.provider = new DefaultFtpClientProvider();
                    return FtpClientProvider.provider;
                }
            });
        }
    }
    
    static boolean access$000() {
        return loadProviderFromProperty();
    }
    
    static boolean access$200() {
        return loadProviderAsService();
    }
    
    static {
        lock = new Object();
        FtpClientProvider.provider = null;
    }
}
